@@ -46,13 +46,13 @@ _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'autogenargs', 'makeargs', 'nice_build', 'jobs',
                 'installprog', 'repos', 'branches', 'noxvfb', 'xvfbargs',
                 'builddir_pattern', 'module_autogenargs', 'module_makeargs',
-                'interact', 'buildscript', 'nonetwork', 'nobuild',
-                'alwaysautogen', 'noinstall', 'makeclean', 'makedistclean',
-                'makecheck', 'module_makecheck', 'system_libdirs',
-                'tinderbox_outputdir', 'sticky_date', 'tarballdir',
+                'module_ninjaargs', 'ninjaargs', 'interact', 'buildscript',
+                'nonetwork', 'nobuild', 'alwaysautogen', 'noinstall',
+                'makeclean', 'makedistclean', 'makecheck', 'module_makecheck',
+                'system_libdirs', 'tinderbox_outputdir', 'sticky_date', 'tarballdir',
                 'pretty_print', 'svn_program', 'makedist', 'makedistcheck',
                 'nonotify', 'notrayicon', 'cvs_program', 'checkout_mode',
-                'copy_dir', 'module_checkout_mode', 'build_policy',
+                'copy_dir', 'export_dir', 'module_checkout_mode', 'build_policy',
                 'trycheckout', 'min_age', 'nopoison', 'module_nopoison',
                 'forcecheck', 'makecheck_advisory', 'quiet_mode',
                 'progress_bar', 'module_extra_env', 'jhbuildbot_master',
@@ -62,10 +62,12 @@ _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'use_local_modulesets', 'ignore_suggests', 'modulesets_dir',
                 'mirror_policy', 'module_mirror_policy', 'dvcs_mirror_dir',
                 'shallow_clone', 'build_targets', 'cmakeargs', 'module_cmakeargs',
+                'mesonargs', 'module_mesonargs',
                 'print_command_pattern', 'static_analyzer',
                 'module_static_analyzer', 'static_analyzer_template',
                 'static_analyzer_outputdir', 'check_sysdeps', 'system_prefix',
-                'help_website', 'conditions', 'extra_prefixes', 'disable_Werror'
+                'help_website', 'conditions', 'extra_prefixes',
+                'disable_Werror', 'xdg_cache_home', 'exit_on_error'
               ]
 
 env_prepends = {}
@@ -194,7 +196,7 @@ class Config:
         if filename:
             try:
                 execfile(filename, config)
-            except Exception, e:
+            except Exception as e:
                 if isinstance(e, FatalError):
                     # raise FatalErrors back, as it means an error in include()
                     # and it will print a traceback, and provide a meaningful
