@@ -21,6 +21,7 @@
 import os
 import logging
 import subprocess
+import sys
 
 from jhbuild.utils import trigger
 from jhbuild.utils import cmds
@@ -59,6 +60,15 @@ class BuildScript:
             if not os.access(self.config.copy_dir, os.R_OK|os.W_OK|os.X_OK):
                 raise FatalError(_('checkout copy dir (%s) must be writable') % self.config.copy_dir)
 
+        if self.config.export_dir and not os.path.exists(self.config.export_dir):
+            try:
+                os.makedirs(self.config.export_dir)
+            except OSError:
+                raise FatalError(
+                        _('checkout export dir (%s) can not be created') % self.config.export_dir)
+            if not os.access(self.config.export_dir, os.R_OK|os.W_OK|os.X_OK):
+                raise FatalError(_('checkout export dir (%s) must be writable') % self.config.export_dir)
+
         self.subprocess_nice_args = []
         if config.nice_build:
             chrt_args = ['chrt', '--idle', '0']
@@ -160,7 +170,7 @@ class BuildScript:
                 try:
                     try:
                         error, altphases = module.run_phase(self, phase)
-                    except SkipToPhase, e:
+                    except SkipToPhase as e:
                         try:
                             num_phase = build_phases.index(e.phase)
                         except ValueError:
@@ -172,6 +182,9 @@ class BuildScript:
                     self._end_phase_internal(module.name, phase, error)
 
                 if error:
+                    if self.config.exit_on_error:
+                        sys.exit(1)
+
                     try:
                         nextphase = build_phases[num_phase+1]
                     except IndexError:
@@ -257,7 +270,7 @@ class BuildScript:
             logging.info(_('Running post-installation trigger script: %r') % (trig.name, ))
             try:
                 self.execute(trig.command())
-            except CommandError, err:
+            except CommandError as err:
                 if isinstance(trig.command(), (str, unicode)):
                     displayed_command = trig.command()
                 else:
