@@ -218,10 +218,11 @@ class GitBranch(Branch):
             raise CommandError(_('Unexpected: Checkoutdir is not a git '
                     'repository:' + self.get_checkoutdir()))
         try:
-            return os.path.basename(
-                    get_output(['git', 'symbolic-ref', '-q', 'HEAD'],
+            full_branch = get_output(['git', 'symbolic-ref', '-q', 'HEAD'],
                             cwd=self.get_checkoutdir(),
-                            extra_env=get_git_extra_env()).strip())
+                            extra_env=get_git_extra_env()).strip()
+            # strip refs/heads/ to get the branch name only
+            return full_branch.replace('refs/heads/', '')
         except CommandError:
             return None
 
@@ -366,9 +367,26 @@ class GitBranch(Branch):
                            % (cmd_desc, stdout))
 
     def _export(self, buildscript):
-        # FIXME: should implement this properly
         self._checkout(buildscript)
 
+        try:
+            output = get_output(['git', 'rev-parse', 'HEAD'],
+                    cwd = self.get_checkoutdir(), get_stderr=False,
+                    extra_env=get_git_extra_env())
+            tag = output.strip()
+        except:
+            tag = 'unknown'
+
+        filename = self.get_module_basename() + '-' + tag + '.zip'
+
+        if self.config.export_dir is not None:
+            path = os.path.join(self.config.export_dir, filename)
+        else:
+            path = os.path.join(self.checkoutroot, filename)
+
+        git_extra_args = {'cwd': self.get_checkoutdir(), 'extra_env': get_git_extra_env()}
+        buildscript.execute(['git', 'archive', '-o', path, 'HEAD'], **git_extra_args)
+
     def _update_submodules(self, buildscript):
         if os.path.exists(os.path.join(self.get_checkoutdir(), '.gitmodules')):
             cmd = ['git', 'submodule', 'init']
@@ -384,11 +402,14 @@ class GitBranch(Branch):
         if self.config.nonetwork:
             return
 
-        # Calculate anew in case a configuration reload changed the mirror root.
+        # Calculate a new in case a configuration reload changed the mirror root.
         mirror_dir = get_git_mirror_directory(self.config.dvcs_mirror_dir,
                 self.checkoutdir, self.unmirrored_module)
 
         if os.path.exists(mirror_dir):
+            buildscript.execute(['git', 'remote', 'set-url', 'origin',
+                    self.unmirrored_module], cwd=mirror_dir,
+                    extra_env=get_git_extra_env())
             buildscript.execute(['git', 'fetch'], cwd=mirror_dir,
                     extra_env=get_git_extra_env())
         else:
@@ -432,15 +453,15 @@ class GitBranch(Branch):
                 raise CommandError(_('Failed to update module as it switched to git (you should check for changes then remove the directory).'))
             raise CommandError(_('Failed to update module (missing .git) (you should check for changes then remove the directory).'))
 
+        if update_mirror:
+            self.update_dvcs_mirror(buildscript)
+
         buildscript.execute(['git', 'remote', 'set-url', 'origin',
                 self.module], **git_extra_args)
 
         buildscript.execute(['git', 'remote', 'update', 'origin'],
                 **git_extra_args)
 
-        if update_mirror:
-            self.update_dvcs_mirror(buildscript)
-
         if self.config.sticky_date:
             self.move_to_sticky_date(buildscript)
 
